package org.openxma.dsl.reference.dao;

import org.openxma.dsl.platform.dao.*;
import java.util.Date;
import org.openxma.dsl.reference.dao.OrderItemDao;
import org.openxma.dsl.reference.model.Order;
import org.openxma.dsl.reference.model.OrderItem;
import org.openxma.dsl.reference.model.Product;

/**
 * Represents a Data Access Object, as defined in the
 * Core J2EE Pattern Catalog (see http://java.sun.com/blueprints/corej2eepatterns/Patterns/DataAccessObject.html),
 * to abstract and encapsulate all access to entity OrderItemDao.
 * @see OrderItemDao
 */
public interface OrderItemDaoGen extends GenericDao<OrderItem,String>, EntityFactory<OrderItem> {
	/**
	 * Sort order constants
	 */
    static final SortOrderSpecification ITEM_NO_ASC = SortOrderSpecification.create("OrderItem.ItemNoAsc").asc("itemNo");
    /**
     * Factory method to create an empty <code>OrderItem</code> entity with an already assigned identifier.
     * The advantage of this approach, coupled with the fact that we're using the id for equals,
     * is that equality test's are immediately possible and therefore independent from the persistence state
     * of the entity. This in contrast to other solutions where auto generated Keys are used
     * which must wait until the object is persisted or differentiate between transient and managed states.
     * @return a new (transient), empty <code>OrderItem</code> entity with an already assigned identifier.
     */
    OrderItem create();
    /**
     * Factory method to create an empty <code>OrderItem</code> taking all not null attributes.
     *
	 * @param order
	 * @param product
	 * @param itemNo
	 * @param quantity
     * @return a new (transient ) <code>OrderItem</code> entity with the given natural identifier and not null attributes set.
     */
    OrderItem create(Order order,Product product,Integer itemNo,Long quantity);
    /**
     * Factory method to create and save an empty <code>OrderItem</code> taking all not null attributes.
     *
	 * @param order
	 * @param product
	 * @param itemNo
	 * @param quantity
     * @return a new (transient) <code>OrderItem</code> entity with the given natural identifier and not null attributes set.
     */
    OrderItem createAndSave(Order order,Product product,Integer itemNo,Long quantity);
    /**
     * Return the persistent instance of the OrderItem entity class
     * with the given natural identifier or <code>null</code> if not found.
     *
	 * @param firstName
	 * @param lastName
	 * @param birthDate
	 * @param orderStoreId
	 * @param orderPlacementDate
	 * @param itemNo
     * @return the persistent OrderItem instance, or <code>null</code> if not found
     */
     OrderItem loadByOrderItemNk(String firstName,String lastName,Date birthDate,String orderStoreId,Date orderPlacementDate,Integer itemNo);
}
