package org.openxma.dsl.reference.xma.order.client;

import java.util.Locale;
import java.util.ResourceBundle;
import org.eclipse.swt.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;
import at.spardat.xma.page.*;
import at.spardat.xma.mdl.*;
import at.spardat.xma.mdl.table.*;
import at.spardat.enterprise.fmt.*;

/**
 * This class implements the generic client part of Page2.
 * It implements the widgets and models of the page.
 *
 */
public abstract class Page2Gen extends org.openxma.dsl.platform.xma.client.DslNotebookPage {
    private Locale genPageLocale = null;
    private ResourceBundle genPageMessages = null;
    private Scaler scaler = null;
    Composite orderItemsTableHeaderW;
    Composite orderItemsTableHeader_set0W;
    Label orderItemsTableHeader_set0_centered;
    Label headerText;
    Table orderItemsTableW;
    at.spardat.xma.mdl.table.TableLayoutManager orderItemsTableWLM;
    ITableWMClient orderItemsTable;
    static final int ORDERITEMSTABLE_ITEMNO = 0;
    static final int ORDERITEMSTABLE_QUANTITY = 1;
    WModel[] widgetModels;
    Control[] widgets;

    /**
     * Constructs the Page2Gen.
     * @param notebook the Notebook this page belongs to.
     */
    public Page2Gen(Notebook notebook) {
        super(notebook,false);
        createModels();
    }

    /*
     * returns the Locale used for this page
    */
    final protected Locale getGenPageLocale() {
        if (genPageLocale == null) {
            genPageLocale = getComponent().getContext().getLocale();
        }
        return genPageLocale;
    }

    /*
     * returns the message resource bundle used for this page
    */
    final protected ResourceBundle getGenPageMessages() {
        if (genPageMessages == null) {
            genPageMessages = ResourceBundle.getBundle("org.openxma.dsl.reference.xma.order.client.Order",getGenPageLocale());
        }
        return genPageMessages;
    }

    /*
     * returns the scaler used for this page
    */
    final protected Scaler getScaler() {
        if (scaler == null) {
            scaler = Scaler.getInstance(getComposite());
        }
        return scaler;
    }

    /**
     * Create and initialize the WidgetModels of the page.
     * Subpages which are not lazy are created too.
     */
    public void createModels() {
        IFmt formatter;
        orderItemsTable = new TableWMClient((short) 0,this,2,TableWMClient.S_NULL|TableWM.S_ONE_WAY);
        formatter = ABcdFmt.getInstance(10, 0, ABcdFmt.DEFAULT, -2147483648, 2147483647, getGenPageLocale());
        ((TableWMClient)orderItemsTable).getColumn(ORDERITEMSTABLE_ITEMNO).setFormatter(formatter);
        formatter = ABcdFmt.getInstance(19, 0, ABcdFmt.DEFAULT, getGenPageLocale());
        ((TableWMClient)orderItemsTable).getColumn(ORDERITEMSTABLE_QUANTITY).setFormatter(formatter);

        widgetModels = new WModel[] { (WModel) orderItemsTable };
        getComponent().registerPageModel(this);
    }

    /**
     * Indicate if the WidgetModels are already created.
     * @return true if the WidgetModels exist.
     */
    public boolean hasModels() {
        return widgetModels != null;
    }

    /**
     * Remove all WidgetModels of the page.
     */
    public void removeWidgetModels() {
        orderItemsTable = null;
        widgetModels = null;
    }

    /**
     * Returns an array of all WidgetModels of this page.
     * @return array of WidgetModels. The returned array is not a copy, therefore
     *          it must not be modified.
     */
    public WModel[] getWModels() {
        return widgetModels;
    }

    /**
     * Replaces the model array generated by guidesigner.
     * May only be called by the runtime. Do not call this method!
     */
    protected void setWModels(WModel[] models) {
        widgetModels=models;
    }

    /**
     * Returns a unique id to identify the type of this Page.
     * @return the typeId
     */
    public short getTypeId() {
        return Order.Page2Class;
    }

    /**
     * Returns the component this page belongs to, casted to its actual type.
     * @return The Component. Never null.
     */
    public Order getTypedComponent() {
        return (Order) getComponent();
    }

    /**
     * Returns an array of all Widgets of this page.
     * @return array of Widgets. The returned array is not a copy, therefore
     *          it must not be modified.
     */
    public Control[] getWidgets() {
        return widgets;
    }

    /**
     * Replaces the widget array generated by guidesigner.
     * May only be called by the runtime. Do not call this method!
     */
    protected void setWidgets(Control[] widgets) {
        this.widgets=widgets;
    }

    /**
     * Create and initialize all SWT-Widgets of the page.
     * It <br>
     * <ul>
     * <li> instantiates the Widgets </li>
     * <li> sets constant texts (Title, Labels, Tooltips) and Images </li>
     * <li> registers the eventadapter </li>
     * <li> defines the Layout (Attachments, Distances, Sizes) </li>
     * <li> sets the focus </li>
     * <li> defines the Taborder </li>
     * </ul>
     */
    protected void createWidgets() {
        EventAdapter adapter = new EventAdapter(this);
        final Scaler scaler = getScaler();

        Composite pageCompositePage2W = getComposite();
        FormLayout layout = new FormLayout();
        layout.marginHeight = scaler.convertYToCurrent(3);
        layout.marginWidth = scaler.convertXToCurrent(3);
        pageCompositePage2W.setLayout(layout);

        orderItemsTableHeaderW = new Composite(pageCompositePage2W,SWT.NONE);
        layout = new FormLayout();
        layout.marginHeight = scaler.convertYToCurrent(3);
        layout.marginWidth = scaler.convertXToCurrent(3);
        orderItemsTableHeaderW.setLayout(layout);
        orderItemsTableHeaderW.setData("WIDGET_ID","orderItemsTableHeaderW");

        orderItemsTableHeader_set0W = new Composite(orderItemsTableHeaderW,SWT.NONE);
        layout = new FormLayout();
        layout.marginHeight = scaler.convertYToCurrent(0);
        layout.marginWidth = scaler.convertXToCurrent(0);
        orderItemsTableHeader_set0W.setLayout(layout);
        orderItemsTableHeader_set0W.setData("WIDGET_ID","orderItemsTableHeader_set0W");

        orderItemsTableHeader_set0_centered = new Label(orderItemsTableHeader_set0W,SWT.LEFT);
        orderItemsTableHeader_set0_centered.setVisible(false);
        orderItemsTableHeader_set0_centered.setData("WIDGET_ID","orderItemsTableHeader_set0_centered");

        headerText = new Label(orderItemsTableHeader_set0W,SWT.LEFT);
        headerText.setText(getGenPageMessages().getString("Page2.headerText"));
        headerText.setData("WIDGET_ID","headerText");

        orderItemsTableW = new Table(pageCompositePage2W,SWT.SINGLE|SWT.BORDER|SWT.FULL_SELECTION
);
        orderItemsTableW.setHeaderVisible(true);
        orderItemsTableW.setLinesVisible(true);
        orderItemsTableW.setData("WIDGET_ID","orderItemsTableW");
        orderItemsTableW.addSelectionListener(adapter);
        TableColumn orderItemsTable_itemNo = new TableColumn(orderItemsTableW,SWT.LEFT);
        orderItemsTable_itemNo.setText(getGenPageMessages().getString("Page2.orderItemsTable_itemNo"));
        orderItemsTable_itemNo.addSelectionListener(adapter);
        TableColumn orderItemsTable_quantity = new TableColumn(orderItemsTableW,SWT.LEFT);
        orderItemsTable_quantity.setText(getGenPageMessages().getString("Page2.orderItemsTable_quantity"));
        orderItemsTable_quantity.addSelectionListener(adapter);
        orderItemsTableW.addMouseListener(adapter);
        Menu orderItemsTableWCM = new Menu(orderItemsTableW.getShell());
        MenuItem orderItemsTableWCMI = new MenuItem(orderItemsTableWCM, SWT.PUSH);
        orderItemsTableWCMI.setText("Copy text");
        orderItemsTableWCMI.addSelectionListener(adapter);
        orderItemsTableW.setMenu(orderItemsTableWCM);
        orderItemsTableWLM = new at.spardat.xma.mdl.table.TableLayoutManager();
        orderItemsTableWLM.manageTable(orderItemsTableW,at.spardat.xma.mdl.table.TableLayoutManager.NONE);
        orderItemsTableWLM.setMinLimit(scaler.convertXToCurrent(10));
        orderItemsTableWLM.setAbsolutWidth(0,scaler.convertXToCurrent(80));
        orderItemsTableWLM.setAbsolutWidth(1,scaler.convertXToCurrent(140));

        widgets = new Control[] { orderItemsTableHeaderW, orderItemsTableHeader_set0W, orderItemsTableHeader_set0_centered, headerText, orderItemsTableW };

        createWidgetsLayout();
        createWidgetsTabOrder();

    }

    /**
     * Create the layout of all SWT-Widgets of the page.
     * (Attachments, Distances, Sizes)
     */
    public void createWidgetsLayout() {
        FormData data;
        final Scaler scaler = getScaler();
        Control lastControl;
        data = new FormData();
        data.left = new FormAttachment(0,100,scaler.convertXToCurrent(0));
        data.right = new FormAttachment(100,100,scaler.convertXToCurrent(0));
        data.bottom = new FormAttachment(0,100,scaler.convertYToCurrent(20));
        orderItemsTableHeaderW.setLayoutData(data);

        lastControl=null;
        data = new FormData();
        data.left = new FormAttachment(0,100,scaler.convertXToCurrent(0));
        data.right = new FormAttachment(100,100,scaler.convertXToCurrent(0));
            data.top = new FormAttachment(0,100,scaler.convertYToCurrent(0));
        orderItemsTableHeader_set0W.setLayoutData(data);
        lastControl=orderItemsTableHeader_set0W;

        data = new FormData();
        data.width = scaler.convertXToCurrent(1);
        data.left = new FormAttachment(0,100,scaler.convertXToCurrent(0));
        data.top = new FormAttachment(0,100,scaler.convertYToCurrent(0));
        data.bottom = new FormAttachment(100,100,scaler.convertYToCurrent(0));
        orderItemsTableHeader_set0_centered.setLayoutData(data);

        data = new FormData();
        data.left = new FormAttachment(0,100,scaler.convertXToCurrent(0));
        data.top = new FormAttachment(orderItemsTableHeader_set0_centered,scaler.convertYToCurrent(0),SWT.CENTER);
        headerText.setLayoutData(data);

        data = new FormData();
        data.left = new FormAttachment(0,100,scaler.convertXToCurrent(0));
        data.right = new FormAttachment(100,100,scaler.convertXToCurrent(0));
        data.top = new FormAttachment(orderItemsTableHeaderW,scaler.convertYToCurrent(0),SWT.BOTTOM);
        data.bottom = new FormAttachment(100,100,scaler.convertYToCurrent(0));
        orderItemsTableW.setLayoutData(data);

    }

    /**
     * Create the tab order of all SWT-Widgets of the page.
     * (sets the focus and defines the tab order)
     */
    protected void createWidgetsTabOrder() {
        Control[] tabOrder;
        final Composite pageCompositePage2W = getComposite();
        tabOrder = new Control[2];
        tabOrder[0] = orderItemsTableHeaderW;
        tabOrder[1] = orderItemsTableW;
        pageCompositePage2W.setTabList(tabOrder);
        tabOrder = new Control[1];
        tabOrder[0] = orderItemsTableHeader_set0W;
        orderItemsTableHeaderW.setTabList(tabOrder);
        tabOrder = new Control[0];
        orderItemsTableHeader_set0W.setTabList(tabOrder);
    }

    /**
     * Remove all Widgets of the page after they have been disposed.
     */
    public void removeWidgets() {
        orderItemsTableHeaderW = null;
        orderItemsTableHeader_set0W = null;
        orderItemsTableHeader_set0_centered = null;
        headerText = null;
        orderItemsTableW = null;
        widgets = null;
    }


    /**
     * Attaches all WidgetModels of the page to there corresponding Widgets.
     */
    public void attachUIImpl() throws AttachmentExceptionClient {
        TableColumn[] cols;
        orderItemsTableW.setData(orderItemsTable.getUIDelegate());
        orderItemsTable.getUIDelegate().attachUI(orderItemsTableW,null);
        cols = orderItemsTableW.getColumns();
        for(int i = 0; i<cols.length; i++) {
            cols[i].setData(orderItemsTable.getUIDelegate());
        }
        if (orderItemsTableW.getMenu() != null) {
           for(MenuItem item : orderItemsTableW.getMenu().getItems()) {
               item.setData(orderItemsTable.getUIDelegate());
           }
        }

    }

    /**
     * This method will be called every time the Page2 becomes visible.
     */
    protected void enter() {
        super.enter();
    }

    /**
     * Calls the clientevent configured for the widget of the event.
     *
     * @param event the SWT-Event that happened.
     * @param type the type of the Event, as defined by the event type constants in class SWT.
     */
    protected void clientEvent(SelectionEvent event,int type) {
        if (event.widget == orderItemsTableW  && type == SWT.Selection) {
            ((OrderEditForm)getParent()).selectOrderItem(event);
        }
    }


    /**
     * Creates the coresponding TabItem
     */
    protected TabItem createTabItem(TabFolder tabfolder1W,int index) {
        TabItem page2W = new TabItem (tabfolder1W, SWT.NULL, index);
        page2W.setText(getGenPageMessages().getString("Page2"));
        page2W.setControl(getComposite());
        page2W.setData(this);

        return page2W;
    }
}
